@@ -33,6 +33,7 @@
 #include <KLocale>
 #include <KMenu>
 #include <KMenuBar>
+#include <KNotificationRestrictions>
 #include <KSqueezedTextLabel>
 #include <KStatusBar>
 #include <KToggleFullScreenAction>
@@ -75,6 +76,7 @@
 #include "loadView.h"
 
 #include <phonon/backendcapabilities.h>
+#include <solid/powermanagement.h>
 
 namespace Dragon {
 
@@ -96,11 +98,13 @@ MainWindow::MainWindow()
         , m_timeLabel( 0 )
         , m_titleLabel( new QLabel( this ) )
         , m_playDialog( 0 )
+        , m_menuToggleAction( 0 )
         , m_stopScreenSaver( 0 )
         , m_stopSleepCookie( -1 )
         , m_stopScreenPowerMgmtCookie( -1 )
         , m_toolbarIsHidden(false)
         , m_statusbarIsHidden(false)
+        , m_menuBarIsHidden(false)
         , m_FullScreenHandler( 0 )
 {
     s_instance = this;
@@ -240,7 +244,7 @@ MainWindow::init()
     if( !kapp->isSessionRestored() ) {
         KCmdLineArgs &args = *KCmdLineArgs::parsedArgs();
         if (args.isSet( "play-dvd" ))
-            engine()->playDvd();
+            playDisc();
         else if (args.count() > 0 ) {
             this->open( args.url( 0 ) );
             args.clear();
@@ -255,6 +259,7 @@ MainWindow::init()
 MainWindow::~MainWindow()
 {
     hide(); //so we appear to have quit, and then sound fades out below
+    releasePowerSave();
     delete videoWindow(); //fades out sound in dtor
 }
 
@@ -264,7 +269,7 @@ void MainWindow::closeEvent (QCloseEvent *event)
     mainWindow()->setWindowState( Qt::WindowNoState );
     statusBar()->setHidden( m_statusbarIsHidden );
     toolBar()->setHidden( m_toolbarIsHidden );
-    menuBar()->setHidden( false );
+    menuBar()->setHidden( m_menuBarIsHidden );
 
     KMainWindow::closeEvent( event );
 }
@@ -301,10 +306,15 @@ MainWindow::setupActions()
     new PlayAction( this, SLOT(play()), ac );
     new VolumeAction( this, SLOT(toggleVolumeSlider(bool)), ac );
 
+    m_menuToggleAction =
+            static_cast<KToggleAction*>(ac->addAction(KStandardAction::ShowMenubar,
+                                                      menuBar(),
+                                                      SLOT(setVisible(bool))));
+
     KAction* playerStop = new KAction( KIcon(QLatin1String( "media-playback-stop" )), i18n("Stop"), ac );
     playerStop->setObjectName( QLatin1String( "stop" ) );
     playerStop->setShortcut( Qt::Key_S );
-    connect( playerStop, SIGNAL(triggered()), engine(), SLOT(stop()) );
+    connect( playerStop, SIGNAL(triggered()), this, SLOT(stop()) );
     addToAc( playerStop )
 
     KToggleAction* mute = new KToggleAction( KIcon(QLatin1String( "player-volume-muted" )), i18nc( "Mute the sound output", "Mute"), ac );
@@ -452,6 +462,7 @@ MainWindow::toggleVolumeSlider( bool show )
     {
         m_volumeSlider = engine()->newVolumeSlider();
         m_volumeSlider->setDisabled ( engine()->isMuted() );
+        m_volumeSlider->setFocus(Qt::PopupFocusReason);
 
         m_muteCheckBox = new QCheckBox();
         m_muteCheckBox->setText( i18nc( "Mute the sound output", "Mute " ) );
@@ -488,6 +499,12 @@ MainWindow::mutedChanged( bool mute )
       }
 }
 
+void MainWindow::stop()
+{
+    engine()->stop();
+    m_mainView->setCurrentWidget(m_loadView);
+}
+
 void
 MainWindow::updateSliders()
 {
@@ -600,23 +617,25 @@ MainWindow::play()
 void
 MainWindow::openFileDialog()
 {
-       QStringList mimeFilter = Phonon::BackendCapabilities::availableMimeTypes();
-        //temporary fixes for MimeTypes that Xine does support but it doesn't return - this is a Xine bug.
-        mimeFilter << QLatin1String( "audio/x-flac");
-        mimeFilter << QLatin1String( "video/mp4" );
-        mimeFilter << QLatin1String( "application/x-cd-image" ); // added for *.iso images
+    QStringList mimeFilter = Phonon::BackendCapabilities::availableMimeTypes();
+    //temporary fixes for MimeTypes that Xine does support but it doesn't return - this is a Xine bug.
+    mimeFilter << QLatin1String( "audio/x-flac");
+    mimeFilter << QLatin1String( "video/mp4" );
+    mimeFilter << QLatin1String( "application/x-cd-image" ); // added for *.iso images
 
-        const KUrl url = KFileDialog::getOpenUrl( KUrl("kfiledialog:///dragonplayer"),mimeFilter.join(QLatin1String( " " ))
-                                        , this, i18n("Select File to Play") );
-        if( url.isEmpty() )
-        {
-            kDebug() << "URL empty in MainWindow::playDialogResult()";
-            return;
-        }
-        else
-        {
-            load( url );
-        }
+    const KUrl url =
+            KFileDialog::getOpenUrl(KGlobalSettings::videosPath(),
+                                    mimeFilter.join(QLatin1String(" ")),
+                                    this, i18n("Select File to Play") );
+    if( url.isEmpty() )
+    {
+        kDebug() << "URL empty in MainWindow::playDialogResult()";
+        return;
+    }
+    else
+    {
+        load( url );
+    }
 }
 
 void
@@ -672,19 +691,24 @@ MainWindow::setFullScreen( bool isFullScreen )
     kDebug() << "Setting full screen to " << isFullScreen;
     mainWindow()->setWindowState( (isFullScreen ? Qt::WindowFullScreen : Qt::WindowNoState ));
 
-	if(isFullScreen)
-	{
-      m_statusbarIsHidden=statusBar()->isHidden();
-	  m_toolbarIsHidden=toolBar()->isHidden();
-	  toolBar()->setHidden( false );
-      statusBar()->setHidden( true );
-	}
-	else
-	{
-      statusBar()->setHidden(m_statusbarIsHidden);
-	  toolBar()->setHidden(m_toolbarIsHidden);
-	}
-    menuBar()->setHidden( isFullScreen );
+    if(isFullScreen)
+    {
+        m_statusbarIsHidden=statusBar()->isHidden();
+        m_toolbarIsHidden=toolBar()->isHidden();
+        m_menuBarIsHidden=menuBar()->isHidden();
+        toolBar()->setHidden( false );
+        statusBar()->setHidden( true );
+        menuBar()->setHidden(true);
+    }
+    else
+    {
+        statusBar()->setHidden(m_statusbarIsHidden);
+        toolBar()->setHidden(m_toolbarIsHidden);
+        menuBar()->setHidden(m_menuBarIsHidden);
+        // In case someone hit the shortcut while being in fullscreen, the action
+        // would be out of sync.
+        m_menuToggleAction->setChecked(!m_menuBarIsHidden);
+    }
     if( m_leftDock )
         m_leftDock->setHidden( isFullScreen );
     // the right dock is handled by the tool bar handler
@@ -780,6 +804,37 @@ MainWindow::keyPressEvent( QKeyEvent *e )
     #undef seek
 }
 
+void
+MainWindow::inhibitPowerSave()
+{
+    if (m_stopSleepCookie == -1)
+        m_stopSleepCookie = Solid::PowerManagement::beginSuppressingSleep(QLatin1String( "watching a film" ));
+    if (m_stopScreenPowerMgmtCookie == -1)
+        m_stopScreenPowerMgmtCookie = Solid::PowerManagement::beginSuppressingScreenPowerManagement(QLatin1String( "watching a film" ));
+    if (!m_stopScreenSaver)
+        m_stopScreenSaver = new KNotificationRestrictions(KNotificationRestrictions::ScreenSaver);
+}
+
+void
+MainWindow::releasePowerSave()
+{
+    //stop supressing sleep
+    if (m_stopSleepCookie != -1) {
+        Solid::PowerManagement::stopSuppressingSleep(m_stopSleepCookie);
+        m_stopSleepCookie = -1;
+    }
+
+    //stop supressing screen power management
+    if (m_stopScreenPowerMgmtCookie != -1) {
+        Solid::PowerManagement::stopSuppressingScreenPowerManagement(m_stopScreenPowerMgmtCookie);
+        m_stopScreenPowerMgmtCookie = -1;
+    }
+
+    //stop disabling screensaver
+    delete m_stopScreenSaver; // It is always 0, I have been careful.
+    m_stopScreenSaver = 0;
+}
+
 QMenu*
 MainWindow::menu( const char *name )
 {
